package JointFeldmanProtocol;

import Communication.MailHandler;
import Communication.Network;

import java.math.BigInteger;
import java.util.*;

/**
 * Created by Tzlil on 3/21/2016.
 */
public class DKGMaliciousUserImpl extends DistributedKeyGenerationUserImpl {

    private final DistributedKeyGeneration maliciousDkg;
    private final Set<Integer> falls;
    public DKGMaliciousUserImpl(DistributedKeyGeneration dkg, Network network, Random random) {
        super(dkg, network);
        this.falls = selectFalls(random);
        this.maliciousDkg = new DistributedKeyGeneration(t,n,randomInt(random),random,dkg.getQ(),g,group,id);
        maliciousDkg.setParties(parties);
    }

    public Set<Integer> selectFalls(Random random){
        ArrayList<Integer> ids = new ArrayList<Integer>();
        for (int i = 1; i<= n ; i++){
            if(i!=id) {
                ids.add(i);
            }
        }
        Set<Integer> falls = new HashSet<Integer>();
        int fallsSize = random.nextInt(ids.size()) + 1;// 1 - (n-1)
        while (falls.size() < fallsSize){
            falls.add(ids.remove(random.nextInt(ids.size())));
        }
        return falls;
    }

    @Override
    public void stage1() {
        dkg.broadcastCommitments(user);
        sendSecrets(); //insteadof dkg.sendSecrets(user);
    }

    @Override
    public void stage3() {
        maliciousDkg.answerAllComplainingPlayers(user);
    }

    @Override
    public void stage4(){
        // do nothing
    }

    private BigInteger randomInt(Random random){
        BigInteger q = dkg.getQ();
        return new BigInteger(q.bitLength(), random).mod(q);
    }

    private void sendSecrets(){
        for (int j = 1; j <= n ; j++){
            if(j != id){
                if(falls.contains(j)){
                    maliciousDkg.sendSecret(user,j);
                }else {
                    dkg.sendSecret(user, j);
                }
            }
        }
    }


}
