package JointFeldmanProtocol;

import Arithmetics.Arithmetic;
import Arithmetics.Fp;
import Arithmetics.Z;
import Communication.Network;
import ShamirSecretSharing.Polynomial;
import ShamirSecretSharing.SecretSharing;
import FeldmanVerifiableSecretSharing.VerifiableSecretSharing;
import UserInterface.DistributedKeyGenerationUser;
import org.factcenter.qilin.primitives.Group;
import org.factcenter.qilin.primitives.concrete.Zpstar;
import org.junit.Before;
import org.junit.Test;

import java.math.BigInteger;
import java.util.*;

/**
 * Created by Tzlil on 2/9/2016.
 */
public class DKGTest {


    DistributedKeyGenerationUser[][] dkgsArrays;
    Thread[][] threadsArrays;
    int tests = 10;
    BigInteger p = BigInteger.valueOf(2903);
    BigInteger q = p.subtract(BigInteger.ONE).divide(BigInteger.valueOf(2));
    BigInteger[] secrets;
    Set<Integer> QUAL = new HashSet<Integer>();
    Arithmetic<BigInteger> arithmetic;
    @Before
    public void settings(){
        Zpstar zpstar = new Zpstar(p);
        Random random = new Random();
        arithmetic = new Fp(q);
        BigInteger g;
        int t = 9;
        int n = 20;
        BigInteger ZERO = zpstar.zero();
        dkgsArrays = new DistributedKeyGenerationUserImpl[tests][n];
        threadsArrays = new Thread[tests][n];
        secrets = new BigInteger[tests];
        DistributedKeyGeneration dkg;
        int abortedStage = 1;
        for (int test = 0; test < tests; test++) {
            do {
                g = zpstar.sample(random);
            } while (!g.equals(ZERO) && !zpstar.multiply(g, q).equals(ZERO));// sample from QRZp*
            secrets[test] = BigInteger.ZERO;
            Network network = new Network(n);
            for (int i = 1; i <= n; i++) {
                BigInteger secret = new BigInteger(q.bitLength(), random).mod(q);
                dkg = new DistributedKeyGeneration(t,n,secret,random,q,g,zpstar,i);

                if(i == n) {
                    dkgsArrays[test][i - 1] = new DKGMaliciousUserImpl(dkg,network,random);//new DKGUserImplAbort(dkg, network, abortedStage);
                }
                else {
                    dkgsArrays[test][i - 1] = new DistributedKeyGenerationUserImpl(dkg, network);
                    QUAL.add(i);
                }
                if (abortedStage > 1 || (abortedStage == 1 && i != n)){
                    secrets[test] = secrets[test].add(secret).mod(q);
                }
                threadsArrays[test][i - 1] = new Thread(dkgsArrays[test][i - 1]);
            }
        }
    }

    public void oneTest(Thread[] threads, DistributedKeyGenerationUser[] dkgs,BigInteger secret) throws Exception {
        for (int i = 0; i < threads.length ; i++){
            threads[i].start();
        }
        for (int i = 0; i < threads.length ; i++){
            threads[i].join();
        }
        int t = dkgs[0].getT();
        int n = dkgs[0].getN();

        Group<BigInteger> zpstar = dkgs[0].getGroup();
        BigInteger g = dkgs[0].getGenerator();

        // got the right public value
        BigInteger publicValue = zpstar.multiply(g,secret);
        for (int i: QUAL){
            if(i != n)
                assert (dkgs[i - 1].getPublicValue().equals(publicValue));
        }

        // assert valid verification values
        BigInteger expected,verification;
        for (int i: QUAL){
            expected = zpstar.multiply(g, dkgs[i - 1].getShare().y);
            verification = VerifiableSecretSharing.verify(i, dkgs[i - 1].getCommitments(),zpstar);
            assert (expected.equals(verification));
        }


        // restore the secret from shares
        ArrayList<Polynomial.Point> sharesList = new ArrayList<Polynomial.Point>();
        Polynomial.Point[] shares = new Polynomial.Point[QUAL.size()];
        for(int i : QUAL){
            sharesList.add(dkgs[i - 1].getShare());
        }
        for (int i = 0; i < shares.length; i ++){
            shares[i] = sharesList.get(i);
        }

        BigInteger calculatedSecret = SecretSharing.restoreSecret(shares,arithmetic);
        assert (calculatedSecret.equals(secret));

    }

    @Test
    public void DKGTest() throws Exception {
        for (int i = 0 ; i < dkgsArrays.length; i ++){
            oneTest(threadsArrays[i],dkgsArrays[i],secrets[i]);
        }
    }
}
